/*! \file sketch.cpp
    \brief Emulation sketch Arduino pour le CESI.
    \author Bertrand Vandeportaele IUT GEII
    \date  14/02/2022
*/
#include "sketch.h"


#include "lib_io_tp.h"

////////////////////////////////////////////////////////////////////
class CStateMachine //déclaration de la classe
{
    //! membres accessibles depuis l'extérieur de la classe, il s'agit de l'interface d'interaction de la classe
public:
    //! Constructeur
    CStateMachine();
    //! méthode pour redémarrer la machine à état
    void reset();
    //! méthode pour cadencer la machine à état (faire 1 coup d'horloge)
    void clock();
    //! méthode manipulateur pour fournir les entrées
    void setEntree(unsigned char entreeval)
    { entree=entreeval;}
    //! méthode manipulateur pour fournir les entrées individuellement
    void setEntreeBit(unsigned char nbit,unsigned char entreeval)
    { entree= (entree&~(1<<nbit)) | ((entreeval&1)<<nbit); } //Serial.println(entree,HEX);
    //! méthode accesseur pour accéder aux sorties
    unsigned char getSortie()
    { return sortie;}
    //! méthode accesseur pour accéder à une sortie individuellement
    unsigned char getSortieBit(unsigned char nbit)
    { return (sortie>>nbit)&1;}
    //! méthode accesseur pour accéder à l'état courant
    unsigned char getEtat()
    { return etat;}

    //! membres privés pour réaliser l'encapsulation: ces attributs sont inacessibles directement depuis l'extérieur de la classe
private:
    //! numéro de l'état actif
    unsigned char etat;
    //! valeur des entrées
    unsigned char entree;
    //! valeur des sorties
    unsigned char sortie;
};
////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////
//Implémentation de la méthode constructeur
CStateMachine::CStateMachine(){
    // à compléter si besoin, allocation etc...
    reset();
}
////////////////////////////////////////////////////////////////////
//Implémentation de la méthode reset, tout ce qui est fait pour (re)démarrer la machine à états
void CStateMachine::reset(){
    etat=0;
    sortie=0; //mettre des valeurs initiales correctes
}
////////////////////////////////////////////////////////////////////
//Implémentation de la méthode clock
void CStateMachine::clock(){
    //variables intérmédiaires pour les entrées
    unsigned char present  =( (entree>>0)&1 ) ^1;
    unsigned char metal    =( (entree>>1)&1 ) ^1;
    unsigned char non_noir =( (entree>>2)&1 ) ^1;
    unsigned char plein    =( (entree>>3)&1 ) ^1;

    //AMZI
    unsigned char moteur=0;
    unsigned char switch1=0;
    unsigned char switch2=0;
    unsigned char stoppeur_retracte=1; //toujours rétracté

    //variables intérmédiaires pour les sorties
    //codage des sorties
    //moteur est une AMZI
    //if ( (etat==0) || (etat==1) || (etat==2) || (etat==3) || (etat==4) || (etat==6) || (etat==7) || (etat==9) || (etat==10) )
    if ( ( (etat==0) || (etat==1) || (etat==2)  || (etat==4) || (etat==7) || (etat==10) ) && (plein==0))
        moteur=1;
    else
        moteur=0;
    if ( (etat==6) || (etat==7) )
        switch1=1;
    else
        switch1=0;
    if ( (etat==3) || (etat==4) )
        switch2=1;
    else
        switch2=0;



    /*unsigned char sortie0;
if (etat==3)
    sortie0=entree1;
else
    sortie0=0;
//Sortie1 est une AMZI
unsigned char sortie1;
if (etat==1)
    sortie1=entree1;
else if ((etat==3)&& (entree0==0))
  sortie1=entree2;
else
    sortie1=0;
//Sortie2 est une AMZE
unsigned char sortie2=(sortie>>2)&1; //récupère la valeur précédente
if ((etat==0) && ( (entree0==1) && (entree1==0)) )
    sortie2=entree2;
//Sortie3 est une AMZE
unsigned char sortie3=(sortie>>3)&1; //récupère la valeur précédente
if ((etat==1) && (entree2==0) )
    sortie3=entree0;
else if ((etat==2) && (entree2==1) && (entree0==0) )
  sortie3=0;
*/

    //codage de l'évolution de l'état
    switch (etat){
    default:
    case 0: if   ( (non_noir==1) && (present==0))
            etat=1;
        else if   (present==1)
            etat=9;
        break;
    case 1: if   ( (metal==1) && (present==0))
            etat=2;
        else if  (present==1)
            etat=6;
        break;
    case 2: if  (present==1)
            etat=3;
        break;
    case 3: if  (plein==0)
            etat=4;
        break;
    case 4: if  (plein==1)
            etat=5;
        break;
    case 5: if  (plein==0)
            etat=0;
        break;
    case 6: if  (plein==0)
            etat=7;
        break;
    case 7: if  (plein==1)
            etat=8;
        break;
    case 8: if  (plein==0)
            etat=0;
        break;
    case 9: if  (plein==0)
            etat=10;
        break;
    case 10: if  (plein==1)
            etat=11;
        break;
    case 11: if  (plein==0)
            etat=0;
        break;
    }
    //pour le premier exercice:
    //sortie=0;
    //reconstruction de sortie à partir des variables intermédiaires
    //sortie=(sortie3<<3) | (sortie2<<2) | (sortie1<<1) | (sortie0<<0) ;
    //stoppeur est en logique inverse, la commande est pour rétracter le stoppeur
    sortie=((stoppeur_retracte)<<3) | (switch2<<2) | (switch1<<1) | (moteur<<0) ;

}
////////////////////////////////////////////////////////////////////

CStateMachine mae;



//////////////////////////////////////////////////////
void setup(){
    init_millis();
    //boucle locale pour simu
    //int r=setupES("127.0.0.1","4242");
    //arduino maison
    //int r=setupES("192.168.1.177","4242");
    //arduino iut gomette bleue
    int r=setupES("172.16.6.60","4242");

    setName("Bertrand Vandeportaele");
    if (r!=0)
        printf("erreur ouverture socket\n");
    printf("Elapsed time: %ld milliseconds\n", millis());
}
//////////////////////////////////////////////////////
void loop(){
    /*
    for (int nbrport=0;nbrport<10;nbrport++){
        writePort(123+nbrport);
        unsigned char e=readPort();
        //std::cout << e << endl;
    }*/
    unsigned int periodiciteTache1=10;
    static unsigned long timerTache1 = millis();
    if (millis() - timerTache1 >= periodiciteTache1) {
        timerTache1 += periodiciteTache1;

        mae.setEntree(readPort());
        mae.clock();
        writePort(mae.getSortie());
        char chaine[100];
        sprintf(chaine,"etat %d", mae.getEtat());
        setMsg(chaine);
    }

/*
    unsigned int periodiciteTache2=1000;
    static unsigned long timerTache2 = millis();
    if (millis() - timerTache2 >= periodiciteTache2) {
        timerTache2 += periodiciteTache2;
        static unsigned int cpt=0;
        cpt++;
        char chaine[100];
        sprintf(chaine,"cpt=%d",cpt);
        printf(chaine);
        setMsg(chaine);
    }
*/




}
